@@ -1,108 +1,112 @@
-#!/usr/bin/env python
-# coding: utf-8
+"""
+Module consolidating common testing functions for checking plotting.
+
+Currently all plotting tests are marked as slow via
+``pytestmark = pytest.mark.slow`` at the module level.
+"""
+
+from __future__ import annotations
 
-import nose
 import os
+from typing import (
+    TYPE_CHECKING,
+    Sequence,
+)
 import warnings
 
-from pandas import DataFrame, Series
-from pandas.compat import zip, iteritems
-from pandas.util.decorators import cache_readonly
-from pandas.types.api import is_list_like
-import pandas.util.testing as tm
-from pandas.util.testing import (ensure_clean,
-                                 assert_is_valid_plot_return_object)
-
 import numpy as np
-from numpy import random
 
-import pandas.tools.plotting as plotting
+from pandas.util._decorators import cache_readonly
+import pandas.util._test_decorators as td
+
+from pandas.core.dtypes.api import is_list_like
+
+import pandas as pd
+from pandas import (
+    DataFrame,
+    Series,
+    to_datetime,
+)
+import pandas._testing as tm
+
+if TYPE_CHECKING:
+    from matplotlib.axes import Axes
 
 
-"""
-This is a common base class used for various plotting tests
-"""
+@td.skip_if_no_mpl
+class TestPlotBase:
+    """
+    This is a common base class used for various plotting tests
+    """
 
-
-def _skip_if_no_scipy_gaussian_kde():
-    try:
-        from scipy.stats import gaussian_kde  # noqa
-    except ImportError:
-        raise nose.SkipTest("scipy version doesn't support gaussian_kde")
-
-
-def _ok_for_gaussian_kde(kind):
-    if kind in ['kde', 'density']:
-        try:
-            from scipy.stats import gaussian_kde  # noqa
-        except ImportError:
-            return False
-    return True
-
-
-@tm.mplskip
-class TestPlotBase(tm.TestCase):
-
-    def setUp(self):
+    def setup_method(self, method):
 
         import matplotlib as mpl
+
+        from pandas.plotting._matplotlib import compat
+
+        self.compat = compat
+
         mpl.rcdefaults()
 
-        self.mpl_le_1_2_1 = plotting._mpl_le_1_2_1()
-        self.mpl_ge_1_3_1 = plotting._mpl_ge_1_3_1()
-        self.mpl_ge_1_4_0 = plotting._mpl_ge_1_4_0()
-        self.mpl_ge_1_5_0 = plotting._mpl_ge_1_5_0()
-        self.mpl_ge_2_0_0 = plotting._mpl_ge_2_0_0()
+        self.start_date_to_int64 = 812419200000000000
+        self.end_date_to_int64 = 819331200000000000
 
-        if self.mpl_ge_1_4_0:
-            self.bp_n_objects = 7
-        else:
-            self.bp_n_objects = 8
-        if self.mpl_ge_1_5_0:
-            # 1.5 added PolyCollections to legend handler
-            # so we have twice as many items.
-            self.polycollection_factor = 2
-        else:
-            self.polycollection_factor = 1
+        self.mpl_ge_2_2_3 = compat.mpl_ge_2_2_3()
+        self.mpl_ge_3_0_0 = compat.mpl_ge_3_0_0()
+        self.mpl_ge_3_1_0 = compat.mpl_ge_3_1_0()
+        self.mpl_ge_3_2_0 = compat.mpl_ge_3_2_0()
 
-        if self.mpl_ge_2_0_0:
-            self.default_figsize = (6.4, 4.8)
-        else:
-            self.default_figsize = (8.0, 6.0)
-        self.default_tick_position = 'left' if self.mpl_ge_2_0_0 else 'default'
-        # common test data
-        from pandas import read_csv
-        path = os.path.join(os.path.dirname(curpath()), 'data', 'iris.csv')
-        self.iris = read_csv(path)
+        self.bp_n_objects = 7
+        self.polycollection_factor = 2
+        self.default_figsize = (6.4, 4.8)
+        self.default_tick_position = "left"
 
         n = 100
         with tm.RNGContext(42):
-            gender = np.random.choice(['Male', 'Female'], size=n)
-            classroom = np.random.choice(['A', 'B', 'C'], size=n)
+            gender = np.random.choice(["Male", "Female"], size=n)
+            classroom = np.random.choice(["A", "B", "C"], size=n)
 
-            self.hist_df = DataFrame({'gender': gender,
-                                      'classroom': classroom,
-                                      'height': random.normal(66, 4, size=n),
-                                      'weight': random.normal(161, 32, size=n),
-                                      'category': random.randint(4, size=n)})
+            self.hist_df = DataFrame(
+                {
+                    "gender": gender,
+                    "classroom": classroom,
+                    "height": np.random.normal(66, 4, size=n),
+                    "weight": np.random.normal(161, 32, size=n),
+                    "category": np.random.randint(4, size=n),
+                    "datetime": to_datetime(
+                        np.random.randint(
+                            self.start_date_to_int64,
+                            self.end_date_to_int64,
+                            size=n,
+                            dtype=np.int64,
+                        )
+                    ),
+                }
+            )
 
         self.tdf = tm.makeTimeDataFrame()
-        self.hexbin_df = DataFrame({"A": np.random.uniform(size=20),
-                                    "B": np.random.uniform(size=20),
-                                    "C": np.arange(20) + np.random.uniform(
-                                        size=20)})
+        self.hexbin_df = DataFrame(
+            {
+                "A": np.random.uniform(size=20),
+                "B": np.random.uniform(size=20),
+                "C": np.arange(20) + np.random.uniform(size=20),
+            }
+        )
 
-    def tearDown(self):
+    def teardown_method(self, method):
         tm.close()
 
     @cache_readonly
     def plt(self):
         import matplotlib.pyplot as plt
+
         return plt
 
     @cache_readonly
     def colorconverter(self):
         import matplotlib.colors as colors
+
         return colors.colorConverter
 
     def _check_legend_labels(self, axes, labels=None, visible=True):
@@ -118,16 +122,37 @@ class TestPlotBase(tm.TestCase):
             expected legend visibility. labels are checked only when visible is
             True
         """
-
         if visible and (labels is None):
-            raise ValueError('labels must be specified when visible is True')
+            raise ValueError("labels must be specified when visible is True")
         axes = self._flatten_visible(axes)
         for ax in axes:
             if visible:
-                self.assertTrue(ax.get_legend() is not None)
+                assert ax.get_legend() is not None
                 self._check_text_labels(ax.get_legend().get_texts(), labels)
             else:
-                self.assertTrue(ax.get_legend() is None)
+                assert ax.get_legend() is None
+
+    def _check_legend_marker(self, ax, expected_markers=None, visible=True):
+        """
+        Check ax has expected legend markers
+
+        Parameters
+        ----------
+        ax : matplotlib Axes object
+        expected_markers : list-like
+            expected legend markers
+        visible : bool
+            expected legend visibility. labels are checked only when visible is
+            True
+        """
+        if visible and (expected_markers is None):
+            raise ValueError("Markers must be specified when visible is True")
+        if visible:
+            handles, _ = ax.get_legend_handles_labels()
+            markers = [handle.get_marker() for handle in handles]
+            assert markers == expected_markers
+        else:
+            assert ax.get_legend() is None
 
     def _check_data(self, xp, rs):
         """
@@ -146,7 +171,7 @@ class TestPlotBase(tm.TestCase):
             rsdata = rsl.get_xydata()
             tm.assert_almost_equal(xpdata, rsdata)
 
-        self.assertEqual(len(xp_lines), len(rs_lines))
+        assert len(xp_lines) == len(rs_lines)
         [check_line(xpl, rsl) for xpl, rsl in zip(xp_lines, rs_lines)]
         tm.close()
 
@@ -162,12 +187,30 @@ class TestPlotBase(tm.TestCase):
             expected visibility
         """
         from matplotlib.collections import Collection
-        if not isinstance(collections,
-                          Collection) and not is_list_like(collections):
+
+        if not isinstance(collections, Collection) and not is_list_like(collections):
             collections = [collections]
 
         for patch in collections:
-            self.assertEqual(patch.get_visible(), visible)
+            assert patch.get_visible() == visible
+
+    def _check_patches_all_filled(
+        self, axes: Axes | Sequence[Axes], filled: bool = True
+    ) -> None:
+        """
+        Check for each artist whether it is filled or not
+
+        Parameters
+        ----------
+        axes : matplotlib Axes object, or its list-like
+        filled : bool
+            expected filling
+        """
+
+        axes = self._flatten_visible(axes)
+        for ax in axes:
+            for patch in ax.patches:
+                assert patch.fill == filled
 
     def _get_colors_mapped(self, series, colors):
         unique = series.unique()
@@ -176,8 +219,9 @@ class TestPlotBase(tm.TestCase):
         mapped = dict(zip(unique, colors))
         return [mapped[v] for v in series.values]
 
-    def _check_colors(self, collections, linecolors=None, facecolors=None,
-                      mapping=None):
+    def _check_colors(
+        self, collections, linecolors=None, facecolors=None, mapping=None
+    ):
         """
         Check each artist has expected line colors and face colors
 
@@ -193,19 +237,21 @@ class TestPlotBase(tm.TestCase):
             Series used for color grouping key
             used for andrew_curves, parallel_coordinates, radviz test
         """
-
-        from matplotlib.lines import Line2D
         from matplotlib.collections import (
-            Collection, PolyCollection, LineCollection
+            Collection,
+            LineCollection,
+            PolyCollection,
         )
+        from matplotlib.lines import Line2D
+
         conv = self.colorconverter
         if linecolors is not None:
 
             if mapping is not None:
                 linecolors = self._get_colors_mapped(mapping, linecolors)
-                linecolors = linecolors[:len(collections)]
+                linecolors = linecolors[: len(collections)]
 
-            self.assertEqual(len(collections), len(linecolors))
+            assert len(collections) == len(linecolors)
             for patch, color in zip(collections, linecolors):
                 if isinstance(patch, Line2D):
                     result = patch.get_color()
@@ -217,15 +263,15 @@ class TestPlotBase(tm.TestCase):
                     result = patch.get_edgecolor()
 
                 expected = conv.to_rgba(color)
-                self.assertEqual(result, expected)
+                assert result == expected
 
         if facecolors is not None:
 
             if mapping is not None:
                 facecolors = self._get_colors_mapped(mapping, facecolors)
-                facecolors = facecolors[:len(collections)]
+                facecolors = facecolors[: len(collections)]
 
-            self.assertEqual(len(collections), len(facecolors))
+            assert len(collections) == len(facecolors)
             for patch, color in zip(collections, facecolors):
                 if isinstance(patch, Collection):
                     # returned as list of np.array
@@ -237,7 +283,7 @@ class TestPlotBase(tm.TestCase):
                     result = tuple(result)
 
                 expected = conv.to_rgba(color)
-                self.assertEqual(result, expected)
+                assert result == expected
 
     def _check_text_labels(self, texts, expected):
         """
@@ -251,15 +297,16 @@ class TestPlotBase(tm.TestCase):
             expected text label, or its list
         """
         if not is_list_like(texts):
-            self.assertEqual(texts.get_text(), expected)
+            assert texts.get_text() == expected
         else:
             labels = [t.get_text() for t in texts]
-            self.assertEqual(len(labels), len(expected))
-            for l, e in zip(labels, expected):
-                self.assertEqual(l, e)
+            assert len(labels) == len(expected)
+            for label, e in zip(labels, expected):
+                assert label == e
 
-    def _check_ticks_props(self, axes, xlabelsize=None, xrot=None,
-                           ylabelsize=None, yrot=None):
+    def _check_ticks_props(
+        self, axes, xlabelsize=None, xrot=None, ylabelsize=None, yrot=None
+    ):
         """
         Check each axes has expected tick properties
 
@@ -276,39 +323,36 @@ class TestPlotBase(tm.TestCase):
             expected yticks rotation
         """
         from matplotlib.ticker import NullFormatter
+
         axes = self._flatten_visible(axes)
         for ax in axes:
-            if xlabelsize or xrot:
+            if xlabelsize is not None or xrot is not None:
                 if isinstance(ax.xaxis.get_minor_formatter(), NullFormatter):
                     # If minor ticks has NullFormatter, rot / fontsize are not
                     # retained
                     labels = ax.get_xticklabels()
                 else:
-                    labels = ax.get_xticklabels() + ax.get_xticklabels(
-                        minor=True)
+                    labels = ax.get_xticklabels() + ax.get_xticklabels(minor=True)
 
                 for label in labels:
                     if xlabelsize is not None:
-                        self.assertAlmostEqual(label.get_fontsize(),
-                                               xlabelsize)
+                        tm.assert_almost_equal(label.get_fontsize(), xlabelsize)
                     if xrot is not None:
-                        self.assertAlmostEqual(label.get_rotation(), xrot)
+                        tm.assert_almost_equal(label.get_rotation(), xrot)
 
-            if ylabelsize or yrot:
+            if ylabelsize is not None or yrot is not None:
                 if isinstance(ax.yaxis.get_minor_formatter(), NullFormatter):
                     labels = ax.get_yticklabels()
                 else:
-                    labels = ax.get_yticklabels() + ax.get_yticklabels(
-                        minor=True)
+                    labels = ax.get_yticklabels() + ax.get_yticklabels(minor=True)
 
                 for label in labels:
                     if ylabelsize is not None:
-                        self.assertAlmostEqual(label.get_fontsize(),
-                                               ylabelsize)
+                        tm.assert_almost_equal(label.get_fontsize(), ylabelsize)
                     if yrot is not None:
-                        self.assertAlmostEqual(label.get_rotation(), yrot)
+                        tm.assert_almost_equal(label.get_rotation(), yrot)
 
-    def _check_ax_scales(self, axes, xaxis='linear', yaxis='linear'):
+    def _check_ax_scales(self, axes, xaxis="linear", yaxis="linear"):
         """
         Check each axes has expected scales
 
@@ -317,16 +361,15 @@ class TestPlotBase(tm.TestCase):
         axes : matplotlib Axes object, or its list-like
         xaxis : {'linear', 'log'}
             expected xaxis scale
-        yaxis :  {'linear', 'log'}
+        yaxis : {'linear', 'log'}
             expected yaxis scale
         """
         axes = self._flatten_visible(axes)
         for ax in axes:
-            self.assertEqual(ax.xaxis.get_scale(), xaxis)
-            self.assertEqual(ax.yaxis.get_scale(), yaxis)
+            assert ax.xaxis.get_scale() == xaxis
+            assert ax.yaxis.get_scale() == yaxis
 
-    def _check_axes_shape(self, axes, axes_num=None, layout=None,
-                          figsize=None):
+    def _check_axes_shape(self, axes, axes_num=None, layout=None, figsize=None):
         """
         Check expected number of axes is drawn in expected layout
 
@@ -336,28 +379,31 @@ class TestPlotBase(tm.TestCase):
         axes_num : number
             expected number of axes. Unnecessary axes should be set to
             invisible.
-        layout :  tuple
+        layout : tuple
             expected layout, (expected number of rows , columns)
         figsize : tuple
             expected figsize. default is matplotlib default
         """
+        from pandas.plotting._matplotlib.tools import flatten_axes
+
         if figsize is None:
             figsize = self.default_figsize
         visible_axes = self._flatten_visible(axes)
 
         if axes_num is not None:
-            self.assertEqual(len(visible_axes), axes_num)
+            assert len(visible_axes) == axes_num
             for ax in visible_axes:
                 # check something drawn on visible axes
-                self.assertTrue(len(ax.get_children()) > 0)
+                assert len(ax.get_children()) > 0
 
         if layout is not None:
-            result = self._get_axes_layout(plotting._flatten(axes))
-            self.assertEqual(result, layout)
+            result = self._get_axes_layout(flatten_axes(axes))
+            assert result == layout
 
-        self.assert_numpy_array_equal(
+        tm.assert_numpy_array_equal(
             visible_axes[0].figure.get_size_inches(),
-            np.array(figsize, dtype=np.float64))
+            np.array(figsize, dtype=np.float64),
+        )
 
     def _get_axes_layout(self, axes):
         x_set = set()
@@ -378,7 +424,9 @@ class TestPlotBase(tm.TestCase):
         axes : matplotlib Axes object, or its list-like
 
         """
-        axes = plotting._flatten(axes)
+        from pandas.plotting._matplotlib.tools import flatten_axes
+
+        axes = flatten_axes(axes)
         axes = [ax for ax in axes if ax.get_visible()]
         return axes
 
@@ -400,17 +448,18 @@ class TestPlotBase(tm.TestCase):
             xerr_count = 0
             yerr_count = 0
             for c in containers:
-                has_xerr = getattr(c, 'has_xerr', False)
-                has_yerr = getattr(c, 'has_yerr', False)
+                has_xerr = getattr(c, "has_xerr", False)
+                has_yerr = getattr(c, "has_yerr", False)
                 if has_xerr:
                     xerr_count += 1
                 if has_yerr:
                     yerr_count += 1
-            self.assertEqual(xerr, xerr_count)
-            self.assertEqual(yerr, yerr_count)
+            assert xerr == xerr_count
+            assert yerr == yerr_count
 
-    def _check_box_return_type(self, returned, return_type, expected_keys=None,
-                               check_ax_title=True):
+    def _check_box_return_type(
+        self, returned, return_type, expected_keys=None, check_ax_title=True
+    ):
         """
         Check box returned type is correct
 
@@ -428,42 +477,43 @@ class TestPlotBase(tm.TestCase):
             Normal ``plot`` doesn't attach ``ax.title``, it must be disabled.
         """
         from matplotlib.axes import Axes
-        types = {'dict': dict, 'axes': Axes, 'both': tuple}
+
+        types = {"dict": dict, "axes": Axes, "both": tuple}
         if expected_keys is None:
             # should be fixed when the returning default is changed
             if return_type is None:
-                return_type = 'dict'
+                return_type = "dict"
 
-            self.assertTrue(isinstance(returned, types[return_type]))
-            if return_type == 'both':
-                self.assertIsInstance(returned.ax, Axes)
-                self.assertIsInstance(returned.lines, dict)
+            assert isinstance(returned, types[return_type])
+            if return_type == "both":
+                assert isinstance(returned.ax, Axes)
+                assert isinstance(returned.lines, dict)
         else:
             # should be fixed when the returning default is changed
             if return_type is None:
                 for r in self._flatten_visible(returned):
-                    self.assertIsInstance(r, Axes)
+                    assert isinstance(r, Axes)
                 return
 
-            self.assertTrue(isinstance(returned, Series))
+            assert isinstance(returned, Series)
 
-            self.assertEqual(sorted(returned.keys()), sorted(expected_keys))
-            for key, value in iteritems(returned):
-                self.assertTrue(isinstance(value, types[return_type]))
+            assert sorted(returned.keys()) == sorted(expected_keys)
+            for key, value in returned.items():
+                assert isinstance(value, types[return_type])
                 # check returned dict has correct mapping
-                if return_type == 'axes':
+                if return_type == "axes":
                     if check_ax_title:
-                        self.assertEqual(value.get_title(), key)
-                elif return_type == 'both':
+                        assert value.get_title() == key
+                elif return_type == "both":
                     if check_ax_title:
-                        self.assertEqual(value.ax.get_title(), key)
-                    self.assertIsInstance(value.ax, Axes)
-                    self.assertIsInstance(value.lines, dict)
-                elif return_type == 'dict':
-                    line = value['medians'][0]
-                    axes = line.axes if self.mpl_ge_1_5_0 else line.get_axes()
+                        assert value.ax.get_title() == key
+                    assert isinstance(value.ax, Axes)
+                    assert isinstance(value.lines, dict)
+                elif return_type == "dict":
+                    line = value["medians"][0]
+                    axes = line.axes
                     if check_ax_title:
-                        self.assertEqual(axes.get_title(), key)
+                        assert axes.get_title() == key
                 else:
                     raise AssertionError
 
@@ -473,91 +523,141 @@ class TestPlotBase(tm.TestCase):
         import matplotlib as mpl
 
         def is_grid_on():
-            xoff = all(not g.gridOn
-                       for g in self.plt.gca().xaxis.get_major_ticks())
-            yoff = all(not g.gridOn
-                       for g in self.plt.gca().yaxis.get_major_ticks())
+            xticks = self.plt.gca().xaxis.get_major_ticks()
+            yticks = self.plt.gca().yaxis.get_major_ticks()
+            # for mpl 2.2.2, gridOn and gridline.get_visible disagree.
+            # for new MPL, they are the same.
+
+            if self.mpl_ge_3_1_0:
+                xoff = all(not g.gridline.get_visible() for g in xticks)
+                yoff = all(not g.gridline.get_visible() for g in yticks)
+            else:
+                xoff = all(not g.gridOn for g in xticks)
+                yoff = all(not g.gridOn for g in yticks)
+
             return not (xoff and yoff)
 
         spndx = 1
         for kind in kinds:
-            if not _ok_for_gaussian_kde(kind):
-                continue
 
             self.plt.subplot(1, 4 * len(kinds), spndx)
             spndx += 1
-            mpl.rc('axes', grid=False)
+            mpl.rc("axes", grid=False)
             obj.plot(kind=kind, **kws)
-            self.assertFalse(is_grid_on())
+            assert not is_grid_on()
 
             self.plt.subplot(1, 4 * len(kinds), spndx)
             spndx += 1
-            mpl.rc('axes', grid=True)
+            mpl.rc("axes", grid=True)
             obj.plot(kind=kind, grid=False, **kws)
-            self.assertFalse(is_grid_on())
+            assert not is_grid_on()
 
-            if kind != 'pie':
+            if kind not in ["pie", "hexbin", "scatter"]:
                 self.plt.subplot(1, 4 * len(kinds), spndx)
                 spndx += 1
-                mpl.rc('axes', grid=True)
+                mpl.rc("axes", grid=True)
                 obj.plot(kind=kind, **kws)
-                self.assertTrue(is_grid_on())
+                assert is_grid_on()
 
                 self.plt.subplot(1, 4 * len(kinds), spndx)
                 spndx += 1
-                mpl.rc('axes', grid=False)
+                mpl.rc("axes", grid=False)
                 obj.plot(kind=kind, grid=True, **kws)
-                self.assertTrue(is_grid_on())
+                assert is_grid_on()
 
-    def _maybe_unpack_cycler(self, rcParams, field='color'):
+    def _unpack_cycler(self, rcParams, field="color"):
         """
-        Compat layer for MPL 1.5 change to color cycle
-
-        Before: plt.rcParams['axes.color_cycle'] -> ['b', 'g', 'r'...]
-        After : plt.rcParams['axes.prop_cycle'] -> cycler(...)
+        Auxiliary function for correctly unpacking cycler after MPL >= 1.5
         """
-        if self.mpl_ge_1_5_0:
-            cyl = rcParams['axes.prop_cycle']
-            colors = [v[field] for v in cyl]
-        else:
-            colors = rcParams['axes.color_cycle']
-        return colors
+        return [v[field] for v in rcParams["axes.prop_cycle"]]
+
+    def get_x_axis(self, ax):
+        return ax._shared_axes["x"] if self.compat.mpl_ge_3_5_0() else ax._shared_x_axes
+
+    def get_y_axis(self, ax):
+        return ax._shared_axes["y"] if self.compat.mpl_ge_3_5_0() else ax._shared_y_axes
 
 
-def _check_plot_works(f, filterwarnings='always', **kwargs):
+def _check_plot_works(f, filterwarnings="always", default_axes=False, **kwargs):
+    """
+    Create plot and ensure that plot return object is valid.
+
+    Parameters
+    ----------
+    f : func
+        Plotting function.
+    filterwarnings : str
+        Warnings filter.
+        See https://docs.python.org/3/library/warnings.html#warning-filter
+    default_axes : bool, optional
+        If False (default):
+            - If `ax` not in `kwargs`, then create subplot(211) and plot there
+            - Create new subplot(212) and plot there as well
+            - Mind special corner case for bootstrap_plot (see `_gen_two_subplots`)
+        If True:
+            - Simply run plotting function with kwargs provided
+            - All required axes instances will be created automatically
+            - It is recommended to use it when the plotting function
+            creates multiple axes itself. It helps avoid warnings like
+            'UserWarning: To output multiple subplots,
+            the figure containing the passed axes is being cleared'
+    **kwargs
+        Keyword arguments passed to the plotting function.
+
+    Returns
+    -------
+    Plot object returned by the last plotting.
+    """
     import matplotlib.pyplot as plt
+
+    if default_axes:
+        gen_plots = _gen_default_plot
+    else:
+        gen_plots = _gen_two_subplots
+
     ret = None
     with warnings.catch_warnings():
         warnings.simplefilter(filterwarnings)
         try:
-            try:
-                fig = kwargs['figure']
-            except KeyError:
-                fig = plt.gcf()
-
+            fig = kwargs.get("figure", plt.gcf())
             plt.clf()
 
-            ax = kwargs.get('ax', fig.add_subplot(211))  # noqa
-            ret = f(**kwargs)
+            for ret in gen_plots(f, fig, **kwargs):
+                tm.assert_is_valid_plot_return_object(ret)
 
-            assert_is_valid_plot_return_object(ret)
-
-            try:
-                kwargs['ax'] = fig.add_subplot(212)
-                ret = f(**kwargs)
-            except Exception:
-                pass
-            else:
-                assert_is_valid_plot_return_object(ret)
-
-            with ensure_clean(return_filelike=True) as path:
+            with tm.ensure_clean(return_filelike=True) as path:
                 plt.savefig(path)
+
+        except Exception as err:
+            raise err
         finally:
             tm.close(fig)
 
         return ret
 
 
+def _gen_default_plot(f, fig, **kwargs):
+    """
+    Create plot in a default way.
+    """
+    yield f(**kwargs)
+
+
+def _gen_two_subplots(f, fig, **kwargs):
+    """
+    Create plot on two subplots forcefully created.
+    """
+    if "ax" not in kwargs:
+        fig.add_subplot(211)
+    yield f(**kwargs)
+
+    if f is pd.plotting.bootstrap_plot:
+        assert "ax" not in kwargs
+    else:
+        kwargs["ax"] = fig.add_subplot(212)
+    yield f(**kwargs)
+
+
 def curpath():
     pth, _ = os.path.split(os.path.abspath(__file__))
     return pth
